/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <media_content.h>
#include <app_debug.h>
#include <app_media.h>
#include <gridmgr.h>
#include <layoutmgr.h>
#include <viewmgr.h>

#include "define.h"
#include "view.h"
#include "data/mediadata.h"
#include "util/listmgr.h"
#include "util/util.h"

/*
   FIXME
#define LIST_MEDIA_COND "media_type=0 OR \
			(media_type=1 AND copyright LIKE \"Unknown\")"
*/
#define LIST_MEDIA_COND "(media_type=0 OR media_type=1) AND copyright LIKE \"Unknown\""

#define TEXT_NOCONTENT "No Photo & Video"

#define GRID_PADDING 6
#define GRID_ITEM_X (200 + GRID_PADDING)
#define GRID_ITEM_Y (200 + GRID_PADDING)
#define GRID_NUM_ITEM 3

#define BOX_PADDING (62 - GRID_PADDING)

struct _priv {
	Evas_Object *base;
	Evas_Object *layout;
	Evas_Object *menu_btn;

	layoutmgr *lmgr;

	struct listmgr *listmgr;
	struct listmgr_data *ldata;

	struct mediadata *md;
	int cur_index;

	Eina_List *media_list;
};

static char *_grid_text_get(void *data, Evas_Object *obj, const char *part)
{
	app_media *am;
	app_media_info *info;
	char buf[32];

	if (!data)
		return NULL;

	am = data;
	info = app_media_get_info(am);
	if (!info) {
		_ERR("failed to get media info");
		return NULL;
	}

	if (info->media_type != MEDIA_CONTENT_TYPE_VIDEO)
		return NULL;

	if (!strcmp(part, PART_ELM_TEXT_PLAYTIME)) {
		util_time_string(buf, sizeof(buf),
					info->video->duration, false);

		return strdup(buf);
	}

	return NULL;
}

static Evas_Object *_grid_content_get(void *data,
			Evas_Object *obj, const char *part)
{
	Evas_Object *image;
	app_media *am;
	app_media_info *info;

	if (!data)
		return NULL;

	am = data;
	info = app_media_get_info(am);
	if (!info) {
		_ERR("failed to get media info");
		return NULL;
	}

	image = NULL;
	if (!strcmp(part, PART_ELM_SWALLOW_THUMBNAIL)) {
		image = util_add_image(obj, info->thumbnail_path);
		if (!image) {
			_ERR("failed to create image object");
			return NULL;
		}

		evas_object_show(image);
	} else if (!strcmp(part, PART_ELM_SWALLOW_VIDEO)) {
		if (info->media_type == MEDIA_CONTENT_TYPE_VIDEO) {
			image = util_add_image(obj, IMAGE_THUMBNAIL_PLAY);
			if (!image) {
				_ERR("failed to create image object");
				return NULL;
			}

			evas_object_show(image);
		}
	} else if (!strcmp(part, PART_ELM_SWALLOW_FAVORITE)) {
		if (!info->favorite)
			return NULL;

		image = util_add_image(obj, IMAGE_THUMBNAIL_FAVORITE);
		if (!image) {
			_ERR("failed to create image object");
			return NULL;
		}

		evas_object_show(image);
	}

	return image;
}

static struct grid_class _gclass = {
	.item_style = STYLE_GRID_GALLERY_ITEM,
	.text_get = _grid_text_get,
	.content_get = _grid_content_get
};

static void _recent_item_selected(struct _priv *priv, app_media *am)
{
	app_media_info *info;
	struct view_update_data vdata;

	info = app_media_get_info(am);
	if (!info) {
		_ERR("failed to get app media info");
		return;
	}

	vdata.list = mediadata_get_medialist(priv->md);
	vdata.index = util_get_media_index_from_id(vdata.list, info->media_id);

	viewmgr_update_view(VIEW_VIEWER, UPDATE_CONTENT, &vdata);
	viewmgr_push_view(VIEW_VIEWER);
}

static void _grid_selected_cb(void *data, Elm_Object_Item *it)
{
	app_media *am;
	struct view_update_data vdata;
	struct _priv *priv;

	if (!data || !it) {
		_ERR("invalid argument");
		return;
	}

	priv = data;

	am = elm_object_item_data_get(it);
	if (!am) {
		_ERR("failed to get app media");
		return;
	}

	vdata.list = mediadata_get_medialist(priv->md);
	vdata.index = util_get_media_index(vdata.list, am);
	priv->cur_index = vdata.index;

	viewmgr_update_view(VIEW_VIEWER, UPDATE_CONTENT, &vdata);
	viewmgr_push_view(VIEW_VIEWER);
}

static struct listmgr_data *_create_listmgr_data(struct _priv *priv)
{
	struct listmgr_data *data;
	struct grid_ops *gops;

	data = calloc(1, sizeof(*data));
	if (!data)
		goto err;

	data->menu_btn = priv->menu_btn;

	data->grid_item_x = GRID_ITEM_X;
	data->grid_item_y = GRID_ITEM_Y;
	data->grid_num_item = GRID_NUM_ITEM;
	data->box_padding = BOX_PADDING;

	gops = calloc(1, sizeof(*gops));
	if (!gops) {
		free(data);
		goto err;
	}

	gops->gclass = &_gclass;
	gops->selected_cb = _grid_selected_cb;
	gops->ops_data = priv;

	data->gops = gops;

	return data;

err:
	_ERR("failed to allocate memory");
	return NULL;
}

static void _update_content_info(struct _priv *priv)
{
	int photo, video;
	char buf[128];

	photo = mediadata_get_media_count(priv->md, E_MEDIA_PHOTO);
	video = mediadata_get_media_count(priv->md, E_MEDIA_VIDEO);

	if (photo < 0 || video < 0) {
		_ERR("failed to get media count");
		return;
	}

	snprintf(buf, sizeof(buf), "%d Photos, %d Videos", photo, video);

	elm_object_part_text_set(priv->layout, PART_CONTENT_INFO, buf);
}

static void _update_content_list(struct _priv *priv)
{
	Eina_List *list;

	if (priv->media_list)
		return;

	list = mediadata_get_list(priv->md, E_LIST_DATE);
	if (!list) {
		elm_object_part_text_set(priv->layout,
				PART_NOCONTENT, TEXT_NOCONTENT);
		return;
	}

	if (!listmgr_update_content_list(priv->listmgr, list))
		_ERR("failed to update list area");

	priv->media_list = list;

	_update_content_info(priv);
}

static bool _create(layoutmgr *lmgr, void *data)
{
	struct listmgr *listmgr;
	struct listmgr_data *ldata;
	struct mediadata *md;
	struct _priv *priv;
	Evas_Object *base, *layout;

	if (!lmgr) {
		_ERR("failed to get layoutmgr");
		return false;
	}

	if (!data) {
		_ERR("invalid argument");
		return false;
	}

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("failed to allocate priv");
		return false;
	}

	priv->menu_btn = (Evas_Object *)data;

	base = layoutmgr_get_base(lmgr);
	if (!base) {
		_ERR("failed to get base object");
		goto err;
	}

	layout = elm_layout_add(base);
	if (!layout) {
		_ERR("failed to create layout");
		goto err;
	}

	if (!elm_layout_file_set(layout, EDJEFILE, GRP_GALLERY_LAYOUT)) {
		_ERR("failed to set layout file");
		goto err2;
	}

	ldata = _create_listmgr_data(priv);
	if (!ldata) {
		_ERR("failed to create listmgr data");
		goto err2;
	}

	listmgr = listmgr_create(layout, (void *)ldata);
	if (!listmgr) {
		_ERR("failed to create listmgr");
		goto err3;
	}

	md = mediadata_create(LIST_MEDIA_COND, E_SOURCE_ALL, E_SORT_DATE);
	if (!md) {
		_ERR("failed to create mediadata");
		listmgr_destroy(listmgr);
		goto err3;
	}

	priv->base = base;
	priv->layout = layout;
	priv->lmgr = lmgr;
	priv->listmgr = listmgr;
	priv->ldata = ldata;
	priv->md = md;

	layoutmgr_set_layout_data(lmgr, LAYOUT_GALLERY, priv);

	if (!listmgr_draw_list_area(listmgr)) {
		_ERR("failed to draw list area");
		mediadata_destroy(md);
		listmgr_destroy(listmgr);
		goto err3;
	}

	return true;

err3:
	free(ldata);
err2:
	evas_object_del(layout);
err:
	free(priv);
	return false;
}

static void _destroy(void *layout_data)
{
	struct _priv *priv;

	if (!layout_data) {
		_ERR("failed to get layout data");
		return;
	}

	priv = layout_data;

	mediadata_free_list(priv->media_list);
	mediadata_destroy(priv->md);

	listmgr_destroy(priv->listmgr);
	free(priv->ldata->gops);
	free(priv->ldata);

	evas_object_del(priv->layout);

	free(priv);
}

static void _show(void *layout_data)
{
	struct _priv *priv;

	if (!layout_data) {
		_ERR("failed to layout data");
		return;
	}

	priv = layout_data;

	evas_object_show(priv->layout);
	elm_object_part_content_set(priv->base,
			PART_THUMBNAIL_AREA, priv->layout);
}

static void _hide(void *layout_data)
{
	struct _priv *priv;

	if (!layout_data) {
		_ERR("failed to get layout data");
		return;
	}

	priv = layout_data;

	evas_object_hide(priv->layout);
	elm_object_part_content_unset(priv->base, PART_THUMBNAIL_AREA);
}

static void _update(void *layout_data, int update_type, void *data)
{
	struct view_update_data *vdata;
	struct _priv *priv;
	bool update;

	if (!layout_data) {
		_ERR("failed to get layout data");
		return;
	}

	priv = layout_data;

	switch (update_type) {
	case UPDATE_CONTENT:
		_update_content_list(priv);
		break;
	case UPDATE_FOCUS:
		if (!data) {
			_ERR("invalid argument");
			return;
		}

		vdata = data;

		if (priv->cur_index != vdata->index)
			update = true;
		else
			update = false;

		listmgr_focus_content_list(priv->listmgr, vdata->index, update);

		break;
	case UPDATE_RECENT:
		if (!data) {
			_ERR("invalid argument");
			return;
		}

		_update_content_list(priv);
		_recent_item_selected(priv, data);
		break;
	case UPDATE_FOCUS_STATE:
		listmgr_set_focus_state(priv->listmgr, EINA_FALSE);
		break;
	default:
		break;
	}
}

static layout_class _lclass = {
	.layout_id = LAYOUT_GALLERY,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.destroy = _destroy,
	.update = _update,
};

layout_class *layout_gallery_get_lclass(void)
{
	return &_lclass;
}
