/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdbool.h>
#include <Elementary.h>
#include <app_debug.h>

#include "util/util.h"

void util_set_alpha_color(Evas_Object *obj, int alpha)
{
	int r, g, b, a;

	evas_object_color_get(obj, &r, &g, &b, &a);

	/* evas should use premultiplied alpha */
	r = r * alpha / 255;
	g = g * alpha / 255;
	b = b * alpha / 255;

	evas_object_color_set(obj, r, g, b, alpha);
}

Evas_Object *util_add_box(Evas_Object *base)
{
	Evas_Object *box;

	if (!base)
		return NULL;

	box = elm_box_add(base);
	if (!box) {
		_ERR("failed to create box object");
		return NULL;
	}

	evas_object_size_hint_align_set(box, 0.0, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(box, 0.0, EVAS_HINT_EXPAND);
	elm_box_horizontal_set(box, EINA_TRUE);

	return box;
}

Evas_Object *util_add_gengrid(Evas_Object *base,
			int item_size_x, int item_size_y)
{
	Evas_Object *grid;

	if (!base)
		return NULL;

	grid = elm_gengrid_add(base);
	if (!grid) {
		_ERR("failed to create gengrid object");
		return NULL;
	}

	evas_object_size_hint_weight_set(grid,
			EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(grid, EVAS_HINT_FILL, EVAS_HINT_FILL);

	elm_gengrid_select_mode_set(grid, ELM_OBJECT_SELECT_MODE_ALWAYS);
	elm_gengrid_multi_select_set(grid, EINA_FALSE);

	elm_gengrid_horizontal_set(grid, EINA_TRUE);

	elm_gengrid_align_set(grid, 0.0, 0.0);

	elm_gengrid_item_size_set(grid,
			elm_config_scale_get() * item_size_x,
			elm_config_scale_get() * item_size_y);

	elm_scroller_policy_set(grid, ELM_SCROLLER_POLICY_OFF,
			ELM_SCROLLER_POLICY_OFF);

	return grid;
}

Evas_Object *util_add_genlist(Evas_Object *base)
{
	Evas_Object *list;

	list = elm_genlist_add(base);
	if (!list) {
		_ERR("failed to adding genlist");
		return NULL;
	}

	evas_object_size_hint_weight_set(list,
			EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	elm_genlist_homogeneous_set(list, EINA_TRUE);
	elm_genlist_select_mode_set(list, ELM_OBJECT_SELECT_MODE_ALWAYS);
	elm_genlist_multi_select_set(list, EINA_FALSE);

	return list;
}

Evas_Object *util_add_image(Evas_Object *base, const char *file)
{
	Evas_Object *image;

	if (!base || !file)
		return NULL;

	image = elm_image_add(base);
	if (!image) {
		_ERR("failed to create image object");
		return NULL;
	}

	elm_image_file_set(image, file, NULL);
	elm_image_aspect_fixed_set(image, EINA_FALSE);

	return image;
}

Evas_Object *util_add_scroller(Evas_Object *base)
{
	Evas_Object *scr;

	if (!base)
		return NULL;

	scr = elm_scroller_add(base);
	if (!scr) {
		_ERR("failed to create scroller object");
		return NULL;
	}

	elm_scroller_policy_set(scr, ELM_SCROLLER_POLICY_OFF,
			ELM_SCROLLER_POLICY_OFF);

	return scr;
}

void util_time_string(char *str, int size, unsigned int ms, bool full)
{
	int sec;
	int h, m, s;

	sec = ms / 1000;

	h = sec / 3600;
	m = (sec % 3600) / 60;
	s = sec % 60;

	if (full) {
		snprintf(str, size, "%02d:%02d:%02d", h, m, s);
	} else {
		if (h)
			snprintf(str, size, "%d:%02d:%02d", h, m, s);
		else
			snprintf(str, size, "%d:%02d", m, s);
	}
}

void util_up_string(char *str)
{
	while (*str) {
		*str = toupper(*str);
		str++;
	}
}

int util_get_media_index(Eina_List *list, void *data)
{
	Eina_List *l;
	int index;
	void *obj;

	index = 0;
	EINA_LIST_FOREACH(list, l, obj) {
		if (data == obj)
			return index;

		index++;
	}

	return -1;
}

int util_get_media_index_from_id(Eina_List *list, const char *id)
{
	Eina_List *l;
	app_media *am;
	app_media_info *info;
	int index;

	index = 0;
	EINA_LIST_FOREACH(list, l, am) {
		info = app_media_get_info(am);
		if (!info)
			continue;

		if (!strcmp(id, info->media_id))
			return index;

		index++;
	}

	return -1;
}

app_media *util_find_media_info(Eina_List *list, const char *id)
{
	Eina_List *l;
	app_media *am;
	app_media_info *info;

	EINA_LIST_FOREACH(list, l, am) {
		info = app_media_get_info(am);
		if (!info)
			continue;

		if (!strcmp(info->media_id, id))
			return am;
	}

	return NULL;
}
