/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <glib.h>
#include <app.h>
#include <Elementary.h>
#include <media_content.h>
#include <app_debug.h>
#include <app_contents.h>
#include <app_media.h>
#include <inputmgr.h>
#include <viewmgr.h>
#include <layoutmgr.h>

#include "define.h"
#include "layout.h"
#include "view.h"

#define TITLE_TEXT "Media Hub"

#define VIDEO_COPYRIGHT "Unknown"

#define BOX_PADDING_SIZE 80

enum _layout_type {
	E_LAYOUT_MOVIE = 0,
	E_LAYOUT_GALLERY,
	E_LAYOUT_MUSIC,
	E_LAYOUT_MAX
};

struct _priv {
	Evas_Object *win;
	Evas_Object *base;
	Evas_Object *btn[E_LAYOUT_MAX];
	Evas_Object *focused_btn;

	layoutmgr *lmgr;

	Eina_List *favorite_list;
	app_media *recent_item;

	int current_layout;
};

struct _menu_item {
	const char *name;
	const char *layout_id;
	layout_class *(*func_get_lclass)(void);
};

static struct _menu_item g_menu_item[E_LAYOUT_MAX] = {
	[E_LAYOUT_MOVIE] = {
		"Movie",
		LAYOUT_MOVIE,
		layout_movie_get_lclass
	},
	[E_LAYOUT_GALLERY] = {
		"Gallery",
		LAYOUT_GALLERY,
		layout_gallery_get_lclass
	},
	[E_LAYOUT_MUSIC] = {
		"Music",
		LAYOUT_MUSIC,
		layout_music_get_lclass
	}
};

static void _set_current_layout(struct _priv *priv, int layout)
{
	layoutmgr_hide_layout(priv->lmgr,
			g_menu_item[priv->current_layout].layout_id);
	layoutmgr_show_layout(priv->lmgr, g_menu_item[layout].layout_id);

	if (priv->focused_btn)
		elm_object_signal_emit(priv->focused_btn,
				SIG_BTN_UNSELECTED, SIG_SOURCE_SRC);

	priv->focused_btn = priv->btn[layout];
	priv->current_layout = layout;
}

static void _key_down_cb(int id, void *data, Evas *e, Evas_Object *obj,
			Evas_Event_Key_Down *ev)
{
	if (!obj || !ev)
		return;

	if (!strcmp(ev->keyname, KEY_ENTER) ||
		!strcmp(ev->keyname, KEY_ENTER_REMOTE))
		elm_object_focus_next(obj, ELM_FOCUS_DOWN);
	else if (!strcmp(ev->keyname, KEY_BACK) ||
		!strcmp(ev->keyname, KEY_BACK_REMOTE))
		ui_app_exit();
}

static void _mouse_move_cb(int id, void *data, Evas *e,
			Evas_Object *obj, Evas_Event_Mouse_Move *ev)
{
	if (!elm_object_focus_get(obj))
		elm_object_focus_set(obj, EINA_TRUE);
}

static void _focused_cb(int id, void *data, Evas_Object *obj,
			Elm_Object_Item *it)
{
	struct _priv *priv;
	int i;

	if (!data) {
		_ERR("failed to get data");
		return;
	}

	priv = data;

	for (i = 0; i < E_LAYOUT_MAX; i++) {
		if (priv->btn[i] == obj)
			break;
	}

	if (i == E_LAYOUT_MAX)
		return;

	layoutmgr_update_layout(priv->lmgr, g_menu_item[i].layout_id,
			UPDATE_FOCUS_STATE, NULL);

	if (priv->focused_btn == obj)
		return;

	_set_current_layout(priv, i);

	layoutmgr_update_layout(priv->lmgr, g_menu_item[i].layout_id,
			UPDATE_CONTENT, NULL);
}

static input_handler handler = {
	.key_down = _key_down_cb,
	.mouse_move = _mouse_move_cb,
	.focused = _focused_cb
};

static bool _check_movie_type(app_media_info *info)
{
	return strcmp(info->video->copyright, VIDEO_COPYRIGHT);
}

static app_media *_get_app_media(const char *id)
{
	media_info_h media;
	app_media *am;
	int r;

	r = media_info_get_media_from_db(id, &media);
	if (r != MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to get media handle");
		return NULL;
	}

	am = app_media_create(media);
	if (!am) {
		_ERR("failed to create app media");
		return NULL;
	}

	return am;
}

static int _get_content_type(app_media_info *info)
{
	int type;

	type = -1;
	switch (info->media_type) {
	case MEDIA_CONTENT_TYPE_IMAGE:
		type = CONTENTS_GALLERY;
		break;
	case MEDIA_CONTENT_TYPE_VIDEO:
		if (_check_movie_type(info))
			type = CONTENTS_MOVIE;
		else
			type = CONTENTS_GALLERY;
		break;
	case MEDIA_CONTENT_TYPE_MUSIC:
		type = CONTENTS_MUSIC;
		break;
	default:
		break;
	}

	return type;
}

static void _favorite_list_foreach(gpointer data, gpointer user_data)
{
	media_info_h media;
	app_media *am;
	Eina_List **l;
	int r;
	char *id;

	if (!data || !user_data) {
		_ERR("invalid argument");
		return;
	}

	id = (char *)data;
	l = (Eina_List **)user_data;

	r = media_info_get_media_from_db(id, &media);
	if (r != MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to get media handle");
		return;
	}

	am = app_media_create(media);
	if (!am) {
		_ERR("failed to create app media");
		media_info_destroy(media);
		return;
	}

	*l = eina_list_append(*l, am);
}

static void _free_favorite_list(Eina_List *list)
{
	app_media *am;

	EINA_LIST_FREE(list, am)
		app_media_destroy(am);
}

static Eina_List *_get_favorite_list(int type)
{
	GList *id_list;
	Eina_List *list;

	id_list = NULL;
	if (app_contents_get_favorite_list(type, &id_list)
			!= APP_CONTENTS_ERROR_NONE) {
		_ERR("failed to get favorite list");
		return NULL;
	}

	list = NULL;
	g_list_foreach(id_list, _favorite_list_foreach, &list);

	app_contents_free_favorite_list(id_list);

	return list;
}

static int _get_media_index(Eina_List *list, const char *id)
{
	Eina_List *l;
	app_media *am;
	app_media_info *info;
	int index;

	index = 0;
	EINA_LIST_FOREACH(list, l, am) {
		info = app_media_get_info(am);
		if (!info)
			continue;

		if (!strcmp(id, info->media_id))
			return index;

		index++;
	}

	return -1;
}

static void _update_favorite_view(struct _priv *priv, const char *id)
{
	app_media *am;
	app_media_info *info;
	struct view_update_data vdata;
	int type;
	int r;

	r = media_content_connect();
	if (r != MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to connect to media content");
		return;
	}

	am = _get_app_media(id);
	if (!am) {
		_ERR("failed to get app media");
		return;
	}

	info = app_media_get_info(am);
	if (!info) {
		_ERR("failed to get app media info");
		app_media_destroy(am);
		return;
	}

	type = _get_content_type(info);

	vdata.list = _get_favorite_list(type);
	vdata.index = _get_media_index(vdata.list, info->media_id);

	if (type == CONTENTS_MOVIE || type == CONTENTS_GALLERY) {
		viewmgr_update_view(VIEW_VIEWER, UPDATE_CONTENT, &vdata);
		viewmgr_push_view(VIEW_VIEWER);
	} else if (type == CONTENTS_MUSIC) {
		viewmgr_update_view(VIEW_MPLAYER, UPDATE_CONTENT, &vdata);
		viewmgr_push_view(VIEW_MPLAYER);
	}

	priv->favorite_list = vdata.list;

	app_media_destroy(am);

	media_content_disconnect();
}

static int _get_layout_type(app_media_info *info)
{
	int type;

	type = E_LAYOUT_MAX;
	switch (info->media_type) {
	case MEDIA_CONTENT_TYPE_IMAGE:
		type = E_LAYOUT_GALLERY;
		break;
	case MEDIA_CONTENT_TYPE_VIDEO:
		if (_check_movie_type(info))
			type = E_LAYOUT_MOVIE;
		else
			type = E_LAYOUT_GALLERY;
		break;
	case MEDIA_CONTENT_TYPE_MUSIC:
		type = E_LAYOUT_MUSIC;
		break;
	default:
		break;
	}

	return type;
}

static void _update_recent_view(struct _priv *priv, const char *id)
{
	app_media *am;
	app_media_info *info;
	int type;
	int r;

	r = media_content_connect();
	if (r != MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to connect to media content");
		return;
	}

	am = _get_app_media(id);
	if (!am) {
		_ERR("failed to get app media");
		return;
	}

	info = app_media_get_info(am);
	if (!info) {
		_ERR("failed to get app media info");
		app_media_destroy(am);
		return;
	}

	type = _get_layout_type(info);

	_set_current_layout(priv, type);

	layoutmgr_update_layout(priv->lmgr, g_menu_item[type].layout_id,
				UPDATE_RECENT, am);

	priv->recent_item = am;

	media_content_disconnect();
}

static bool _draw_title(struct _priv *priv)
{
	if (!priv)
		return false;

	elm_object_part_text_set(priv->base, PART_TITLE, TITLE_TEXT);

	return true;
}

static bool _draw_menu_btn(struct _priv *priv)
{
	Evas_Object *box, *btn;
	int i;

	if (!priv)
		return false;

	box = elm_box_add(priv->base);
	if (!box) {
		_ERR("failed to create box object");
		return false;
	}

	elm_box_horizontal_set(box, EINA_TRUE);
	elm_box_padding_set(box, BOX_PADDING_SIZE, 0);

	evas_object_size_hint_weight_set(box,
			EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);

	for (i = 0; i < E_LAYOUT_MAX; i++) {
		btn = elm_button_add(box);
		if (!btn) {
			_ERR("failed to create button object");
			return false;
		}

		elm_object_style_set(btn, STYLE_BTN_MENU);
		elm_object_text_set(btn, g_menu_item[i].name);

		elm_box_pack_end(box, btn);

		evas_object_show(btn);

		inputmgr_add_callback(btn, 0, &handler, priv);

		priv->btn[i] = btn;
	}

	evas_object_show(box);

	elm_object_part_content_set(priv->base, PART_MENU_AREA, box);

	elm_object_focus_next_object_set(priv->btn[E_LAYOUT_MAX - 1],
				priv->btn[0], ELM_FOCUS_RIGHT);
	elm_object_focus_next_object_set(priv->btn[0],
				priv->btn[E_LAYOUT_MAX - 1], ELM_FOCUS_LEFT);

	return true;
}

static bool _draw_items(struct _priv *priv)
{
	if (!priv)
		return false;

	if (!_draw_title(priv))
		return false;

	if (!_draw_menu_btn(priv))
		return false;

	return true;
}

static Evas_Object *_create(Evas_Object *win, void *data)
{
	struct _priv *priv;
	Evas_Object *base;
	layoutmgr *lmgr;
	int i;

	if (!win) {
		_ERR("failed to get win object");
		return NULL;
	}

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("failed to allocate priv");
		return NULL;
	}

	base = elm_layout_add(win);
	if (!base) {
		_ERR("failed to create base object");
		free(priv);
		return NULL;
	}

	elm_layout_file_set(base, EDJEFILE, GRP_BASE_VIEW);

	evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND,
				EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, base);

	priv->win = win;
	priv->base = base;

	if (!_draw_items(priv)) {
		_ERR("failed to draw items");
		free(priv);
		return NULL;
	}

	lmgr = layoutmgr_create(base);
	for (i = 0; i < E_LAYOUT_MAX; i++)
		layoutmgr_add_layout(lmgr, g_menu_item[i].func_get_lclass(),
					priv->btn[i]);

	priv->lmgr = lmgr;

	viewmgr_set_view_data(VIEW_BASE, priv);

	elm_object_focus_set(priv->btn[0], EINA_TRUE);

	return base;
}

static void _show(void *view_data)
{
	struct _priv *priv;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = view_data;

	evas_object_show(priv->base);
}

static void _hide(void *view_data)
{
	struct _priv *priv;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = view_data;

	evas_object_hide(priv->base);
}

static void _update(void *view_data, int update_type, void *data)
{
	struct _priv *priv;
	struct view_update_data *vdata;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	if (!data) {
		_ERR("invalid argument");
		return;
	}

	priv = view_data;

	switch (update_type) {
	case UPDATE_FOCUS:
	case UPDATE_PLAY_INFO:
		vdata = data;

		layoutmgr_update_layout(priv->lmgr,
				g_menu_item[priv->current_layout].layout_id,
				update_type, vdata);
		break;
	case UPDATE_FAVORITE:
		_update_favorite_view(priv, data);
		break;
	case UPDATE_RECENT:
		_update_recent_view(priv, data);
		break;
	default:
		break;
	}
}

static void _pause(void *view_data)
{
	viewmgr_update_view(VIEW_MPLAYER, UPDATE_PLAYER, NULL);
}

static void _destroy(void *view_data)
{
	struct _priv *priv;
	int i;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = view_data;

	for (i = 0; i < E_LAYOUT_MAX; i++)
		inputmgr_remove_callback(priv->btn[i], &handler);

	layoutmgr_remove_layout(priv->lmgr, LAYOUT_MOVIE);
	layoutmgr_remove_layout(priv->lmgr, LAYOUT_GALLERY);
	layoutmgr_remove_layout(priv->lmgr, LAYOUT_MUSIC);

	layoutmgr_destroy(priv->lmgr);

	if (priv->favorite_list)
		_free_favorite_list(priv->favorite_list);

	app_media_destroy(priv->recent_item);

	evas_object_del(priv->base);

	free(priv);
}

static view_class _vclass = {
	.view_id = VIEW_BASE,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.update = _update,
	.pause = _pause,
	.destroy = _destroy,
};

view_class *view_base_get_vclass(void)
{
	return &_vclass;
}
